@@ -20,9 +20,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#ifndef S_SPLINT_S
 #include <ctype.h>
-#endif
 #include "core/alphabet.h"
 #include "core/chardef.h"
 #include "core/cstr_api.h"
@@ -764,7 +762,7 @@ static inline char converttoprettysymbol(const GtAlphabet *alphabet,
 }
 
 void gt_alphabet_decode_seq_to_cstr(const GtAlphabet *alphabet, char *dest,
-                                      const GtUchar *src, GtUword len)
+                                    const GtUchar *src, GtUword len)
 {
   GtUword i;
 
