@@ -1,4 +1,5 @@
 /*
+  Copyright (C) 2015 Annika Seidel, annika.seidel@studium.uni-hamburg.de
   Copyright (c) 2007-2009 Gordon Gremme <gordon@gremme.org>
   Copyright (c) 2007-2008 Center for Bioinformatics, University of Hamburg
 
@@ -21,19 +22,13 @@
 #include "core/minmax.h"
 #include "extended/affinealign.h"
 
-typedef enum {
-  R,
-  D,
-  I
-} Edge;
-
 typedef struct {
   GtUword Rdist,
                 Ddist,
                 Idist;
-  Edge Redge,
-       Dedge,
-       Iedge;
+  AffineAlignEdge Redge,
+                  Dedge,
+                  Iedge;
 } AffinealignDPentry;
 
 static GtUword infadd(GtUword inf, GtUword s)
@@ -44,14 +39,15 @@ static GtUword infadd(GtUword inf, GtUword s)
 }
 
 static void affinealign_fill_table(AffinealignDPentry **dptable,
-                                   const char *u, GtUword ulen,
-                                   const char *v, GtUword vlen,
-                                   int replacement_cost, int gap_opening,
-                                   int gap_extension)
+                                   const GtUchar *u, GtUword ulen,
+                                   const GtUchar *v, GtUword vlen,
+                                   int matchcost, int mismatchcost,
+                                   int gap_opening, int gap_extension)
 {
   GtUword i, j, Rvalue, Dvalue, Ivalue, minvalue;
   int rcost;
-  gt_assert(dptable && u && ulen && v && vlen);
+  gt_assert(dptable && u && v);
+  /*gt_assert(ulen && vlen);*/
   for (i = 0; i <= ulen; i++) {
     for (j = 0; j <= vlen; j++) {
       if (!i && !j) {
@@ -64,7 +60,7 @@ static void affinealign_fill_table(AffinealignDPentry **dptable,
         if (!i || !j)
           dptable[i][j].Rdist = ULONG_MAX;
         else {
-          rcost  = (u[i-1] == v[j-1]) ? 0 : replacement_cost;
+          rcost  = (u[i-1] == v[j-1]) ? matchcost : mismatchcost;
           Rvalue = infadd(dptable[i-1][j-1].Rdist, rcost);
           Dvalue = infadd(dptable[i-1][j-1].Ddist, rcost);
           Ivalue = infadd(dptable[i-1][j-1].Idist, rcost);
@@ -73,11 +69,11 @@ static void affinealign_fill_table(AffinealignDPentry **dptable,
           dptable[i][j].Rdist = minvalue;
           /* set backtracing edge */
           if (Rvalue == minvalue)
-            dptable[i][j].Redge = R;
+            dptable[i][j].Redge = Affine_R;
           else if (Dvalue == minvalue)
-            dptable[i][j].Redge = D;
+            dptable[i][j].Redge = Affine_D;
           else /* Ivalue == minvalue */
-            dptable[i][j].Redge = I;
+            dptable[i][j].Redge = Affine_I;
         }
         /* compute A_affine(i,j,D) */
         if (!i)
@@ -91,11 +87,11 @@ static void affinealign_fill_table(AffinealignDPentry **dptable,
           dptable[i][j].Ddist = minvalue;
           /* set backtracing edge */
           if (Rvalue == minvalue)
-            dptable[i][j].Dedge = R;
+            dptable[i][j].Dedge = Affine_R;
           else if (Dvalue == minvalue)
-            dptable[i][j].Dedge = D;
+            dptable[i][j].Dedge = Affine_D;
           else /* Ivalue == minvalue */
-            dptable[i][j].Dedge = I;
+            dptable[i][j].Dedge = Affine_I;
         }
         /* compute A_affine(i,j,I) */
         if (!j)
@@ -109,73 +105,78 @@ static void affinealign_fill_table(AffinealignDPentry **dptable,
           dptable[i][j].Idist = minvalue;
           /* set backtracing edge */
           if (Rvalue == minvalue)
-            dptable[i][j].Iedge = R;
+            dptable[i][j].Iedge = Affine_R;
           else if (Dvalue == minvalue)
-            dptable[i][j].Iedge = D;
+            dptable[i][j].Iedge = Affine_D;
           else /* Ivalue == minvalue */
-            dptable[i][j].Iedge = I;
+            dptable[i][j].Iedge = Affine_I;
         }
       }
     }
   }
 }
 
-static void affinealign_traceback(GtAlignment *a, AffinealignDPentry **dptable,
+static void affinealign_traceback(GtAlignment *a,
+                                  AffinealignDPentry * const *dptable,
                                   GtUword i, GtUword j)
 {
   GtUword minvalue;
-  Edge edge;
+  AffineAlignEdge edge;
   gt_assert(a && dptable);
   /* determine min{A_affine(m,n,x) | x in {R,D,I}} */
   minvalue = MIN3(dptable[i][j].Rdist, dptable[i][j].Ddist,
                   dptable[i][j].Idist);
   if (dptable[i][j].Rdist == minvalue)
-    edge = R;
+    edge = Affine_R;
   else if (dptable[i][j].Ddist == minvalue)
-    edge = D;
+    edge = Affine_D;
   else /* dptable[i][j].Idist == minvalue */
-    edge = I;
+    edge = Affine_I;
   /* backtracing */
   while (i > 0 || j > 0) {
     switch (edge) {
-      case R:
+      case Affine_R:
         gt_assert(dptable[i][j].Rdist != ULONG_MAX);
         gt_alignment_add_replacement(a);
         edge = dptable[i][j].Redge;
-        /* gt_assert(i && j); */
+        gt_assert(i > 0 && j > 0);
         i--;
         j--;
         break;
-      case D:
+      case Affine_D:
         gt_alignment_add_deletion(a);
         edge = dptable[i][j].Dedge;
         gt_assert(i);
         i--;
         break;
-      case I:
+      case Affine_I:
         gt_alignment_add_insertion(a);
         edge = dptable[i][j].Iedge;
         gt_assert(j);
         j--;
         break;
+      default:
+        gt_assert(false);
     }
   }
 }
 
-GtAlignment* gt_affinealign(const char *u, GtUword ulen,
-                            const char *v, GtUword vlen,
-                            int replacement_cost, int gap_opening_cost,
-                            int gap_extension_cost)
+/* SK: why use int costs here and GtWord-costs in linearalign? */
+
+GtAlignment* gt_affinealign(const GtUchar *u, GtUword ulen,
+                            const GtUchar *v, GtUword vlen,
+                            int matchcost, int mismatchcost,
+                            int gap_opening_cost, int gap_extension_cost)
 {
   AffinealignDPentry **dptable;
-  GtAlignment *a;
-  gt_assert(u && ulen && v && vlen);
+  GtAlignment *align;
+
+  gt_assert(u && v);
   gt_array2dim_malloc(dptable, ulen+1, vlen+1);
-  affinealign_fill_table(dptable, u, ulen, v, vlen, replacement_cost,
+  affinealign_fill_table(dptable, u, ulen, v, vlen, matchcost, mismatchcost,
                          gap_opening_cost, gap_extension_cost);
-  a = gt_alignment_new_with_seqs((const GtUchar *) u, ulen, (const GtUchar *) v,
-                                 vlen);
-  affinealign_traceback(a, dptable, ulen, vlen);
+  align = gt_alignment_new_with_seqs(u, ulen,  v, vlen);
+  affinealign_traceback(align, dptable, ulen, vlen);
   gt_array2dim_delete(dptable);
-  return a;
+  return align;
 }
