@@ -1,4 +1,5 @@
 /*
+  Copyright (C) 2015 Annika Seidel, annika.seidel@studium.uni-hamburg.de
   Copyright (c) 2007 Gordon Gremme <gordon@gremme.org>
   Copyright (c) 2007 Center for Bioinformatics, University of Hamburg
 
@@ -20,10 +21,18 @@
 
 #include "extended/alignment.h"
 
+typedef enum {
+  Affine_R,
+  Affine_D,
+  Affine_I,
+  Affine_X /* unknown */
+} AffineAlignEdge;
+
 /* (globally) align u and v (affine gap costs) and return one optimal
    GtAlignment */
-GtAlignment* gt_affinealign(const char *u, GtUword ulen,
-                       const char *v, GtUword vlen, int replacement_cost,
-                       int gap_opening_cost, int gap_extension_cost);
+GtAlignment* gt_affinealign(const GtUchar *u, GtUword ulen,
+                            const GtUchar *v, GtUword vlen,
+                            int matchcost, int mismatchcost,
+                            int gap_opening_cost, int gap_extension_cost);
 
 #endif
