@@ -1,4 +1,5 @@
 /*
+  Copyright (c) 2015 Annika Seidel <annika.seidel@studium.uni-hamburg.de>
   Copyright (c) 2006-2007 Gordon Gremme <gordon@gremme.org>
   Copyright (c) 2006-2007 Center for Bioinformatics, University of Hamburg
 
@@ -43,6 +44,8 @@ void         gt_alignment_set_urange(GtAlignment *alignment, GtRange range);
 void         gt_alignment_set_vrange(GtAlignment *alignment, GtRange range);
 /* can be either match, mismatch, <GtMultieoplist> handles them as match! */
 void         gt_alignment_add_replacement(GtAlignment *alignment);
+void         gt_alignment_add_replacement_multi(GtAlignment *alignment,
+                                                GtUword num);
 void         gt_alignment_add_deletion(GtAlignment *alignment);
 void         gt_alignment_add_insertion(GtAlignment *alignment);
 /* NOT the length of the alignment, but the number of distinct stored elements
@@ -55,20 +58,44 @@ void         gt_alignment_remove_last(GtAlignment *alignment);
 void         gt_alignment_reset(GtAlignment *alignment);
 /* returns unit cost */
 GtUword      gt_alignment_eval(const GtAlignment*);
+GtUword gt_alignment_eval_generic(bool mapped,const GtAlignment *alignment);
 GtWord       gt_alignment_eval_with_score(const GtAlignment *alignment,
                                           GtWord matchscore,
                                           GtWord mismatchscore,
                                           GtWord gapscore);
+GtWord       gt_alignment_eval_with_affine_score(const GtAlignment *alignment,
+                                           const GtWord matchscore,
+                                           const GtWord mismatchscore,
+                                           const GtWord gap_opening,
+                                           const GtWord gap_extension);
 /* print alignment to <fp>. This will break the lines after width characters */
 void         gt_alignment_show(const GtAlignment *alignment, FILE *fp,
                                unsigned int width);
 void         gt_alignment_show_with_mapped_chars(const GtAlignment *alignment,
                                                  const GtUchar *characters,
                                                  GtUchar wildcardshow,
-                                                 FILE *fp);
+                                                 FILE *fp,
+                                                 unsigned int width);
+GtUchar *gt_alignment_buffer_new(unsigned int width);
+void gt_alignment_buffer_delete(GtUchar *buffer);
+void gt_alignment_show_generic(GtUchar *buffer,
+                               const GtAlignment *alignment,
+                               FILE *fp,
+                               unsigned int width,
+                               const GtUchar *characters,
+                               GtUchar wildcardshow);
+void gt_alignment_exact_show(GtUchar *buffer,
+                             const GtUchar *sequence,
+                             GtUword seqlen,
+                             FILE *fp,
+                             unsigned int width,
+                             const GtUchar *characters);
 void         gt_alignment_show_multieop_list(const GtAlignment *alignment,
                                              FILE *fp);
 int          gt_alignment_unit_test(GtError *err);
 void         gt_alignment_delete(GtAlignment *alignment);
 
+void gt_alignment_clone(const GtAlignment *alignment_from,
+                              GtAlignment *alignment_to);
+
 #endif
