@@ -9,6 +9,7 @@
 #include "core/assert_api.h"
 #include "core/divmodmul.h"
 #include "core/unused_api.h"
+#include "core/arraydef.h"
 #include "core/ma_api.h"
 #include "core/types_api.h"
 #include "ft-front-generation.h"
@@ -78,17 +79,17 @@ static size_t front_trace_size(const Fronttrace *front_trace)
 void front_trace_reset(Fronttrace *front_trace,
                        GT_UNUSED GtUword sumseqlen)
 {
-  if (front_trace != NULL)
-  {
 #ifdef WITHDISTRIBUTION
-    double space_per_pos = (double) front_trace_size(front_trace)/sumseqlen;
-
-    distribution_add(front_trace->space_per_pos_dist,
-                     (GtUword) (space_per_pos * 100.0));
+  double space_per_pos = (double) front_trace_size(front_trace)/sumseqlen;
 #endif
-    front_trace->backref_nextfree = 0;
-    front_trace->gen_nextfree = 0;
-  }
+
+  gt_assert(front_trace != NULL);
+#ifdef WITHDISTRIBUTION
+  distribution_add(front_trace->space_per_pos_dist,
+                   (GtUword) (space_per_pos * 100.0));
+#endif
+  front_trace->backref_nextfree = 0;
+  front_trace->gen_nextfree = 0;
 }
 
 void front_trace_delete(Fronttrace *front_trace)
@@ -116,10 +117,7 @@ void front_trace_delete(Fronttrace *front_trace)
 void front_trace_add_gen(Fronttrace *front_trace,GtUword trimleft,
                          GtUword valid)
 {
-  if (front_trace == NULL)
-  {
-    return;
-  }
+  gt_assert (front_trace != NULL);
   if (front_trace->gen_nextfree >= front_trace->gen_allocated)
   {
     front_trace->gen_allocated
@@ -157,10 +155,7 @@ void front_trace_add_gen(Fronttrace *front_trace,GtUword trimleft,
 void front_trace_add_trace(Fronttrace *front_trace,uint8_t backreference,
                            unsigned int lcs)
 {
-  if (front_trace == NULL)
-  {
-    return;
-  }
+  gt_assert (front_trace != NULL);
   if (front_trace->backref_nextfree >= front_trace->backref_allocated)
   {
     front_trace->backref_allocated
@@ -208,6 +203,139 @@ static GtUword valid_total_fronts(const FrontGeneration *gen_table,
   return valid_total;
 }
 
+#ifndef OUTSIDE_OF_GT
+void front_trace_multireplacement(GtArrayuint8_t *eoplist,GtUword repnum)
+{
+  const GtUword addamount = 128;
+  while (true)
+  {
+    if (repnum <= FT_EOPCODE_MAXREPLACEMENT)
+    {
+      gt_assert(repnum > 0);
+      GT_STOREINARRAY(eoplist,uint8_t,addamount,(uint8_t) (repnum - 1));
+      break;
+    }
+    GT_STOREINARRAY(eoplist,uint8_t,addamount,
+                    (uint8_t) (FT_EOPCODE_MAXREPLACEMENT - 1));
+    repnum -= FT_EOPCODE_MAXREPLACEMENT;
+  }
+}
+
+void front_trace2eoplist(GtArrayuint8_t *eoplist,
+                         const Fronttrace *front_trace,
+                         const Polished_point *pp,
+                         GtUword ulen,
+                         GtUword vlen)
+{
+  GtUword distance, localoffset, globaloffset, remainingvalidfronts,
+          totalrunlength = 0, trimleft;
+  GtWord diagonal;
+  unsigned int row, lcs;
+  uint8_t trace, preferred_eop = FT_EOP_REPLACEMENT;
+
+  gt_assert(front_trace != NULL && front_trace->gen_nextfree > 0 && pp != NULL);
+  localoffset = polished_point2offset(front_trace,pp);
+  remainingvalidfronts = valid_total_fronts(front_trace->gen_table,
+                                            pp->distance,
+                                            front_trace->gen_nextfree);
+  gt_assert(remainingvalidfronts <= front_trace->backref_nextfree);
+  globaloffset = front_trace->backref_nextfree - remainingvalidfronts;
+  distance = pp->distance;
+  diagonal = (GtWord) pp->alignedlen - (GtWord) GT_MULT2(pp->row);
+  trace = front_trace->backref_table[globaloffset + localoffset].bits;
+  lcs = front_trace->backref_table[globaloffset + localoffset].lcs;
+  row = pp->row;
+  trimleft = pp->trimleft;
+  gt_assert(distance < front_trace->gen_nextfree);
+  while (distance > 0)
+  {
+    GtUword nextrowadd;
+    GtWord base_diagonal;
+
+    if (lcs > 0)
+    {
+      front_trace_multireplacement(eoplist,lcs);
+    }
+    if (trace & preferred_eop)
+    {
+      totalrunlength++;
+      if (preferred_eop == FT_EOP_REPLACEMENT)
+      {
+        nextrowadd = 1;
+      } else
+      {
+        if (preferred_eop == FT_EOP_INSERTION)
+        {
+          gt_assert(-(GtWord) ulen < diagonal);
+          diagonal--;
+          nextrowadd = 0;
+        } else
+        {
+          gt_assert(preferred_eop == FT_EOP_DELETION);
+          gt_assert(diagonal < (GtWord) vlen);
+          diagonal++;
+          nextrowadd = 1;
+        }
+      }
+    } else
+    {
+      if (trace & FT_EOP_REPLACEMENT)
+      {
+        preferred_eop = FT_EOP_REPLACEMENT;
+        nextrowadd = 1;
+      } else
+      {
+        if (trace & FT_EOP_INSERTION)
+        {
+          gt_assert(-(GtWord) ulen < diagonal);
+          diagonal--;
+          preferred_eop = FT_EOP_INSERTION;
+          nextrowadd = 0;
+        } else
+        {
+          gt_assert(trace & FT_EOP_DELETION);
+          gt_assert(diagonal < (GtWord) vlen);
+          diagonal++;
+          preferred_eop = FT_EOP_DELETION;
+          nextrowadd = 1;
+        }
+      }
+    }
+    if (preferred_eop == FT_EOP_DELETION)
+    {
+      GT_STOREINARRAY(eoplist,uint8_t,ulen + vlen,FT_EOPCODE_DELETION);
+    } else
+    {
+      if (preferred_eop == FT_EOP_INSERTION)
+      {
+        GT_STOREINARRAY(eoplist,uint8_t,ulen + vlen,FT_EOPCODE_INSERTION);
+      } else
+      {
+        GT_STOREINARRAY(eoplist,uint8_t,ulen + vlen,0);
+      }
+    }
+    gt_assert(trimleft >=
+              (GtUword) front_trace->gen_table[distance].trimleft_diff);
+    trimleft -= (GtUword) front_trace->gen_table[distance].trimleft_diff;
+    distance--;
+    base_diagonal = (GtWord) trimleft - (GtWord) distance;
+    gt_assert(base_diagonal <= diagonal);
+    gt_assert(diagonal <
+              base_diagonal + (GtWord) front_trace->gen_table[distance].valid);
+    localoffset = (GtUword) (diagonal - base_diagonal);
+    gt_assert((GtUword) front_trace->gen_table[distance].valid
+              <= globaloffset);
+    globaloffset -= (GtUword) front_trace->gen_table[distance].valid;
+    gt_assert(row >= lcs + nextrowadd);
+    row -= lcs + nextrowadd;
+    trace = front_trace->backref_table[globaloffset + localoffset].bits;
+    lcs = front_trace->backref_table[globaloffset + localoffset].lcs;
+  }
+  /*printf("avg runlength=%.2f\n",(double) pp->distance/totalrunlength);*/
+  gt_assert(globaloffset + localoffset == 0 && trace == 0);
+}
+#endif
+
 #define DEBUG
 #ifdef DEBUG
 static void check_diagonal_run(GT_UNUSED const GtUchar *useq,
