@@ -27,13 +27,13 @@
 #include "gth/gt_gthbssmtrain.h"
 #include "gth/gt_gthmkbssmfiles.h"
 #include "tools/gt_compressedbits.h"
-#include "tools/gt_condenser.h"
 #include "tools/gt_consensus_sa.h"
 #include "tools/gt_extracttarget.h"
 #include "tools/gt_gdiffcalc.h"
 #include "tools/gt_guessprot.h"
 #include "tools/gt_idxlocali.h"
 #include "tools/gt_kmer_database.h"
+#include "tools/gt_linspace_align.h"
 #include "tools/gt_magicmatch.h"
 #include "tools/gt_mergeesa.h"
 #include "tools/gt_paircmp.h"
@@ -68,7 +68,6 @@ static void* gt_dev_arguments_new(void)
   gt_toolbox_add(dev_toolbox, "regioncov", gt_regioncov);
   gt_toolbox_add(dev_toolbox, "trieins", gt_trieins);
   gt_toolbox_add_tool(dev_toolbox, "compbits", gt_compressedbits());
-  gt_toolbox_add_tool(dev_toolbox, "condenser", gt_condenser());
   gt_toolbox_add_tool(dev_toolbox, "consensus_sa", gt_consensus_sa_tool());
   gt_toolbox_add_tool(dev_toolbox, "extracttarget", gt_extracttarget());
   gt_toolbox_add_tool(dev_toolbox, "gdiffcalc", gt_gdiffcalc());
@@ -76,6 +75,7 @@ static void* gt_dev_arguments_new(void)
   gt_toolbox_add_tool(dev_toolbox, "gthbssmtrain", gt_gthbssmtrain());
   gt_toolbox_add_tool(dev_toolbox, "idxlocali", gt_idxlocali());
   gt_toolbox_add_tool(dev_toolbox, "kmer_database", gt_kmer_database());
+  gt_toolbox_add_tool(dev_toolbox, "linspace_align", gt_linspace_align());
   gt_toolbox_add_tool(dev_toolbox, "magicmatch", gt_magicmatch());
   gt_toolbox_add_tool(dev_toolbox, "parsexrf", gt_parsexrf());
   gt_toolbox_add_tool(dev_toolbox, "readreads", gt_readreads());
