@@ -8,14 +8,6 @@ allfiles = ["Atinsert.fna",
             "TTT-small.fna",
             "trna_glutamine.fna"]
 
-repfindtestfiles=["Duplicate.fna",
-                  "Wildcards.fna",
-                  "hs5hcmvcg.fna",
-                  "humhbb.fna",
-                  "mpomtcg.fna",
-                  "at1MB",
-                  "ychrIII.fna"]
-
 def makegreedyfwdmatcall(queryfile,indexarg,ms)
   prog=""
   if ms
@@ -77,54 +69,6 @@ def createandcheckgreedyfwdmat(reffile,queryfile)
   checkgreedyfwdmat(queryfile,true)
 end
 
-def addfilepath(filename)
-  if filename == 'Duplicate.fna' or filename == 'at1MB'
-    return "#{$testdata}#{filename}"
-  else
-    return "#{$gttestdata}DNA-mix/Grumbach.fna/#{filename}"
-  end
-end
-
-def determineminlength(reffile)
-  if reffile == 'at1MB'
-    return 22
-  else
-    return 14
-  end
-end
-
-def checkrepfind(reffile)
-  reffilepath=addfilepath(reffile)
-  run_test("#{$bin}gt suffixerator -algbds 3 31 80 -db " +
-           "#{reffilepath} -indexname sfxidx -dna -suf -tis -lcp -ssp -pl",
-           :maxtime => 320)
-  minlength = determineminlength(reffile)
-  run_test("#{$bin}gt repfind -l #{minlength} -ii sfxidx", :maxtime => 600)
-  resultfile="#{$gttestdata}repfind-result/#{reffile}.result"
-  run "cmp -s #{last_stdout} #{resultfile}"
-  run_test("#{$bin}gt repfind -l #{minlength} -r -ii sfxidx", :maxtime => 600)
-  resultfile="#{$gttestdata}repfind-result/#{reffile}-r.result"
-  run "cmp -s #{last_stdout} #{resultfile}"
-end
-
-def checkrepfindwithquery(reffile,queryfile)
-  reffilepath=addfilepath(reffile)
-  queryfilepath=addfilepath(queryfile)
-  idxname=reffile + "-idx"
-  run_test "#{$bin}gt suffixerator -algbds 3 31 80 -db " +
-           "#{reffilepath} -indexname #{idxname} -dna -suf -tis -lcp -ssp -pl"
-  run_test("#{$bin}gt repfind -l 15 -ii #{idxname} -q #{queryfilepath}",
-           :maxtime => 600)
-  # run "sort #{last_stdout}"
-  #run "/Users/kurtz/bin-ops/i686-apple-darwin/mkvtree.x -indexname mkv-idx " +
-  #    "-allout -v -pl -dna -db #{reffilepath}"
-  #run "/Users/kurtz/bin-ops/i686-apple-darwin/vmatch-mini.x 15 mkv-idx " +
-  #    "#{queryfilepath}"
-  #run "sed -e '/^#/d' #{last_stdout} | sort"
-  # run "#{$scriptsdir}repfind-cmp.rb #{last_stdout} #{$gttestdata}repfind-result/#{reffile}-#{queryfile}.result"
-  run "cmp -s #{last_stdout} #{$gttestdata}repfind-result/#{reffile}-#{queryfile}.result"
-end
-
 Name "gt paircmp"
 Keywords "gt_paircmp"
 Test do
@@ -154,51 +98,6 @@ Test do
   run_test "#{$bin}gt dev patternmatch -samples 10000 -ii sfx"
 end
 
-Name "gt repfind small"
-Keywords "gt_repfind"
-Test do
-  run_test "#{$bin}gt suffixerator -db #{$testdata}Atinsert.fna " +
-           "-indexname sfx -dna -tis -suf -lcp -ssp -pl"
-  run_test "#{$bin}gt repfind -l 8 -ii sfx"
-  run "grep -v '^#' #{last_stdout}"
-  run "diff -w #{last_stdout} #{$testdata}repfind-8-Atinsert.txt"
-  run_test "#{$bin}gt repfind -scan -l 8 -ii sfx"
-  run "grep -v '^#' #{last_stdout}"
-  run "diff -w #{last_stdout} #{$testdata}repfind-8-Atinsert.txt"
-  run_test "#{$bin}gt repfind -samples 10 -l 6 -ii sfx",:maxtime => 600
-  run "#{$bin}gt repfind -samples 1000 -l 6 -ii sfx",:maxtime => 600
-end
-
-if $gttestdata then
-  Name "gt repfind extend at1MB"
-  Keywords "gt_repfind_extend"
-  Test do
-    run_test "#{$bin}gt suffixerator -db #{$testdata}at1MB " +
-             "-indexname sfx -dna -tis -suf -lcp"
-    run_test "#{$bin}gt repfind -l 20 -extendxdrop -ii sfx"
-    run "diff #{last_stdout} #{$testdata}repfind-20-extend.txt"
-    run_test "#{$bin}gt repfind -l 20 -extendxdrop -ii sfx -q " +
-             "#{$testdata}/U89959_genomic.fas"
-    run "diff #{last_stdout} #{$testdata}repfind-20-query-extend.txt"
-  end
-  repfindtestfiles.each do |reffile|
-    Name "gt repfind #{reffile}"
-    Keywords "gt_repfind gttestdata"
-    Test do
-      checkrepfind(reffile)
-    end
-    repfindtestfiles.each do |queryfile|
-      if reffile != queryfile
-        Name "gt repfind #{reffile} versus #{queryfile}"
-        Keywords "gt_repfind gttestdata"
-        Test do
-          checkrepfindwithquery(reffile,queryfile)
-        end
-      end
-    end
-  end
-end
-
 allfiles.each do |reffile|
   allfiles.each do |queryfile|
     if queryfile != reffile
@@ -265,7 +164,7 @@ allfiles.each do |reffile|
 end
 
 Name "gt matstat/uniquesub at1MB U8"
-Keywords "gt_greedyfwdmat gt_matstat gt_uniquesub gttestdata"
+Keywords "gt_greedyfwdmat"
 Test do
   createandcheckgreedyfwdmat("#{$testdata}at1MB",
                              "#{$testdata}U89959_genomic.fas")
