@@ -162,7 +162,6 @@ require 'gt_bed_to_gff3_include'
 require 'gt_cds_include'
 require 'gt_chseqids_include'
 require 'gt_condenseq_include'
-require 'gt_condenser_include'
 require 'gt_consensus_sa_include'
 require 'gt_csa_include'
 require 'gt_csr_include.rb'
@@ -182,6 +181,7 @@ require 'gt_include'
 require 'gt_inlineseq_include'
 require 'gt_interfeat_include'
 require 'gt_kmer_database_include'
+require 'gt_linspace_align_include'
 require 'gt_loccheck_include'
 require 'gt_ltrdigest_include'
 require 'gt_ltrharvest_include'
@@ -235,6 +235,7 @@ require 'gt_cmpprjfiles_include'
 require 'gt_env_options_include'
 require 'gt_extractseq_include'
 require 'gt_idxsearch_include'
+require 'gt_repfind_include'
 require 'gt_mergeesa_include'
 require 'gt_packedindex_include'
 require 'gt_sortbench_include'
